import React, { Component } from "react";
import { LatLngTuple } from "leaflet";
import "leaflet/dist/leaflet.css";
export declare type Circle = {
    center: LatLngTuple;
    radius: number;
};
export declare type Rectangle = [LatLngTuple, LatLngTuple];
export declare type PointSeries = LatLngTuple[];
export declare type PickerMode = "points" | "circles" | "rectangles" | "polylines" | "polygons";
export declare type IViewport = {
    center: [number, number];
    zoom: number;
};
export declare type ILocationPickerProps = Readonly<typeof defaultProps>;
declare const defaultProps: {
    geoserver: boolean;
    geoURL: string;
    geoLayer: string;
    mapStyle: React.CSSProperties;
    bindMap: boolean;
    startPort: "auto" | "default" | IViewport;
    overlayAll: boolean;
    showControls: boolean;
    showInputs: boolean;
    useDynamic: boolean;
    precision: number;
    pointMode: PointMode | undefined;
    circleMode: CircleMode | undefined;
    rectangleMode: RectangleMode | undefined;
    polylineMode: PointSeriesMode | undefined;
    polygonMode: PointSeriesMode | undefined;
};
export declare type PointMode = {
    control?: {
        values: LatLngTuple[];
        onClick?: (point: LatLngTuple) => void;
        onRemove?: (point: LatLngTuple) => void;
    };
    banner: boolean;
};
export declare type CircleMode = {
    control?: {
        values: Circle[];
        onClick?: (point: LatLngTuple) => void;
        onRemove?: (circle: Circle) => void;
    };
    banner: boolean;
};
export declare type RectangleMode = {
    control?: {
        values: Rectangle[];
        onClick?: (point: LatLngTuple) => void;
        onRemove?: (rectangle: Rectangle) => void;
    };
    banner: boolean;
};
export declare type PointSeriesMode = {
    control?: {
        values: PointSeries[];
        onClick?: (point: LatLngTuple) => void;
        onAdd?: () => void;
        onRemove?: (pointSeries: PointSeries) => void;
    };
    banner: boolean;
};
declare type ILocationPickerState = Readonly<typeof defaultState>;
declare const defaultState: {
    throttleTimer: number;
    lat: number;
    lng: number;
    hoverPosition: [number, number];
    pickerMode: PickerMode;
    points: [number, number][];
    circles: Circle[];
    circleCenter: [number, number] | null;
    rectangles: [[number, number], [number, number]][];
    rectangleCorner: [number, number] | null;
    polylines: [number, number][][];
    partialPolyline: [number, number][];
    polygons: [number, number][][];
    partialPolygon: [number, number][];
};
export default class LocationPicker extends Component<ILocationPickerProps, ILocationPickerState> {
    constructor(props: ILocationPickerProps);
    static defaultProps: {
        geoserver: boolean;
        geoURL: string;
        geoLayer: string;
        mapStyle: React.CSSProperties;
        bindMap: boolean;
        startPort: "auto" | "default" | IViewport;
        overlayAll: boolean;
        showControls: boolean;
        showInputs: boolean;
        useDynamic: boolean;
        precision: number;
        pointMode: PointMode | undefined;
        circleMode: CircleMode | undefined;
        rectangleMode: RectangleMode | undefined;
        polylineMode: PointSeriesMode | undefined;
        polygonMode: PointSeriesMode | undefined;
    };
    render(): JSX.Element;
    componentDidMount: () => void;
    private renderTileLayer;
    private renderBanner;
    private renderModeControl;
    private renderOverlays;
    private calculatePoints;
    private calculateCircles;
    private calculateRectangles;
    private calculatePolylines;
    private calculatePolygons;
    private renderInputs;
    private getInitialPickerMode;
    private handleClick;
    private handleMouseMove;
    private inputChange;
    private changeMode;
    private addPolyline;
    private addPolygon;
    private removeObject;
}
export {};
