"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const react_leaflet_1 = require("react-leaflet");
const leaflet_1 = require("leaflet");
const react_leaflet_control_1 = __importDefault(require("react-leaflet-control"));
const Banner_1 = __importDefault(require("./Banner"));
const Overlays_1 = __importDefault(require("./Overlays"));
const utils_1 = require("./utils");
require("leaflet/dist/leaflet.css");
const defaultProps = {
    geoserver: false,
    geoURL: "",
    geoLayer: "",
    mapStyle: { height: 300, width: "auto" },
    bindMap: true,
    startPort: "default",
    overlayAll: true,
    showControls: true,
    showInputs: true,
    useDynamic: true,
    precision: 6,
    pointMode: undefined,
    circleMode: undefined,
    rectangleMode: undefined,
    polylineMode: undefined,
    polygonMode: undefined
};
const defaultState = {
    throttleTimer: new Date().getTime(),
    lat: 0,
    lng: 0,
    hoverPosition: [0, 0],
    pickerMode: "points",
    points: [],
    circles: [],
    circleCenter: null,
    rectangles: [],
    rectangleCorner: null,
    polylines: [],
    partialPolyline: [],
    polygons: [],
    partialPolygon: []
};
const mapBounds = [
    [-90, -180],
    [90, 180]
];
const defaultViewport = {
    center: [30, 0],
    zoom: 2
};
class LocationPicker extends react_1.Component {
    constructor(props) {
        super(props);
        this.componentDidMount = () => {
            // @ts-ignore
            const map = this.refs.map;
            const { startPort, pointMode, circleMode, rectangleMode, polylineMode, polygonMode } = this.props;
            if (startPort === "auto") {
                let points = [];
                if (pointMode && pointMode.control)
                    points = Array.from(pointMode.control.values);
                if (circleMode && circleMode.control)
                    circleMode.control.values.forEach(circ => utils_1.calculateCircleBox(circ).forEach(point => points.push(point)));
                if (rectangleMode && rectangleMode.control)
                    rectangleMode.control.values.forEach(rect => rect.forEach(point => points.push(point)));
                if (polylineMode && polylineMode.control)
                    polylineMode.control.values.forEach(polyl => polyl.forEach(point => points.push(point)));
                if (polygonMode && polygonMode.control)
                    polygonMode.control.values.forEach(polyg => polyg.forEach(point => points.push(point)));
                if (points.length === 1) {
                    map.leafletElement.setView(points[0], 6);
                }
                else if (points.length > 1) {
                    const bounds = new leaflet_1.LatLngBounds(points);
                    const zoom = map.leafletElement.getBoundsZoom(bounds);
                    map.leafletElement.setView(bounds.getCenter(), zoom);
                }
            }
            else if (startPort !== "default") {
                map.leafletElement.setView(startPort.center, startPort.zoom);
            }
        };
        this.renderTileLayer = () => {
            const { geoserver, geoURL, geoLayer } = this.props;
            if (geoserver) {
                return (react_1.default.createElement(react_leaflet_1.WMSTileLayer, { url: geoURL, layers: geoLayer, attribution: "Geoserver" }));
            }
            else {
                return (react_1.default.createElement(react_leaflet_1.TileLayer, { url: "https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png", attribution: '&amp;copy <a href="http://osm.org/copyright">OpenStreetMap</a> contributors' }));
            }
        };
        this.renderBanner = () => {
            const { pointMode, circleMode, rectangleMode, polylineMode, polygonMode } = this.props;
            const bannerProps = {};
            if (pointMode && pointMode.banner) {
                if (pointMode.control) {
                    bannerProps.points = pointMode.control.values;
                    bannerProps.pointRemoval = pointMode.control.onRemove;
                }
                else {
                    bannerProps.points = this.state.points;
                    bannerProps.pointRemoval = this.removeObject("points");
                }
            }
            if (circleMode && circleMode.banner) {
                if (circleMode.control) {
                    bannerProps.circles = circleMode.control.values;
                    bannerProps.circleRemoval = circleMode.control.onRemove;
                }
                else {
                    bannerProps.circles = this.state.circles;
                    bannerProps.circleRemoval = this.removeObject("circles");
                }
            }
            if (rectangleMode && rectangleMode.banner) {
                if (rectangleMode.control) {
                    bannerProps.rectangles = rectangleMode.control.values;
                    bannerProps.rectangleRemoval = rectangleMode.control.onRemove;
                }
                else {
                    bannerProps.rectangles = this.state.rectangles;
                    bannerProps.rectangleRemoval = this.removeObject("rectangles");
                }
            }
            if (polylineMode && polylineMode.banner) {
                if (polylineMode.control) {
                    bannerProps.polylines = polylineMode.control.values;
                    bannerProps.polylineRemoval = polylineMode.control.onRemove;
                }
                else {
                    bannerProps.polylines = this.state.polylines;
                    bannerProps.polylineRemoval = this.removeObject("polylines");
                }
            }
            if (polygonMode && polygonMode.banner) {
                if (polygonMode.control) {
                    bannerProps.polygons = polygonMode.control.values;
                    bannerProps.polygonRemoval = polygonMode.control.onRemove;
                }
                else {
                    bannerProps.polygons = this.state.polygons;
                    bannerProps.polygonRemoval = this.removeObject("polygons");
                }
            }
            return react_1.default.createElement(Banner_1.default, Object.assign({}, bannerProps));
        };
        this.renderModeControl = () => {
            const { pointMode, circleMode, rectangleMode, polylineMode, polygonMode, showControls } = this.props;
            if (!showControls)
                return null;
            const buttons = [];
            if (pointMode) {
                buttons.push(react_1.default.createElement("button", { key: "point", onClick: this.changeMode("points") }, "Point"));
            }
            if (circleMode) {
                buttons.push(react_1.default.createElement("button", { key: "circle", onClick: this.changeMode("circles") }, "Circle"));
            }
            if (rectangleMode) {
                buttons.push(react_1.default.createElement("button", { key: "rectangle", onClick: this.changeMode("rectangles") }, "Rectangle"));
            }
            if (polylineMode) {
                buttons.push(react_1.default.createElement("button", { key: "polyline", onClick: this.changeMode("polylines") }, "Lines"));
                if (this.state.partialPolyline.length > 0) {
                    if (polylineMode.control) {
                        if (polylineMode.control.onAdd) {
                            buttons.push(react_1.default.createElement("button", { key: "polylineAdd", onClick: polylineMode.control.onAdd }, "Add"));
                        }
                    }
                    else {
                        buttons.push(react_1.default.createElement("button", { key: "polylineAdd", onClick: this.addPolyline }, "Add"));
                    }
                }
            }
            if (polygonMode) {
                buttons.push(react_1.default.createElement("button", { key: "polygon", onClick: this.changeMode("polygons") }, "Polygon"));
                if (this.state.partialPolygon.length > 0) {
                    if (polygonMode.control) {
                        if (polygonMode.control.onAdd) {
                            buttons.push(react_1.default.createElement("button", { key: "polygonAdd", onClick: polygonMode.control.onAdd }, "Add"));
                        }
                    }
                    else {
                        buttons.push(react_1.default.createElement("button", { key: "polygonAdd", onClick: this.addPolygon }, "Add"));
                    }
                }
            }
            return react_1.default.createElement(react_leaflet_control_1.default, { position: "topright" }, buttons);
        };
        this.renderOverlays = () => {
            const opts = {
                points: this.calculatePoints(),
                circles: this.calculateCircles(),
                rectangles: this.calculateRectangles(),
                polylines: this.calculatePolylines(),
                polygons: this.calculatePolygons()
            };
            return react_1.default.createElement(Overlays_1.default, Object.assign({}, opts));
        };
        this.calculatePoints = () => {
            const { pointMode, overlayAll } = this.props;
            const { pickerMode, points } = this.state;
            if (pointMode && (overlayAll || pickerMode === "points")) {
                return pointMode.control ? pointMode.control.values : points;
            }
            return [];
        };
        this.calculateCircles = () => {
            const { circleMode, overlayAll, useDynamic } = this.props;
            const { pickerMode, circles, circleCenter, hoverPosition } = this.state;
            if (circleMode && (overlayAll || pickerMode === "circles")) {
                if (circleMode.control)
                    return circleMode.control.values;
                const movingCircle = useDynamic && circleCenter
                    ? {
                        center: circleCenter,
                        radius: utils_1.calculateRadius(circleCenter, hoverPosition)
                    }
                    : null;
                return movingCircle ? circles.concat([movingCircle]) : circles;
            }
            return [];
        };
        this.calculateRectangles = () => {
            const { rectangleMode, overlayAll, useDynamic } = this.props;
            const { pickerMode, rectangles, rectangleCorner, hoverPosition } = this.state;
            if (rectangleMode && (overlayAll || pickerMode === "rectangles")) {
                if (rectangleMode.control)
                    return rectangleMode.control.values;
                const movingRectangle = useDynamic && rectangleCorner ? [rectangleCorner, hoverPosition] : null;
                return movingRectangle
                    ? rectangles.concat([movingRectangle])
                    : rectangles;
            }
            return [];
        };
        this.calculatePolylines = () => {
            const { polylineMode, overlayAll, useDynamic } = this.props;
            const { pickerMode, polylines, partialPolyline, hoverPosition } = this.state;
            if (polylineMode && (overlayAll || pickerMode === "polylines")) {
                if (polylineMode.control)
                    return polylineMode.control.values;
                return useDynamic
                    ? polylines.concat([partialPolyline.concat([hoverPosition])])
                    : polylines.concat([partialPolyline]);
            }
            return [];
        };
        this.calculatePolygons = () => {
            const { polygonMode, overlayAll, useDynamic } = this.props;
            const { pickerMode, polygons, partialPolygon, hoverPosition } = this.state;
            if (polygonMode && (overlayAll || pickerMode === "polygons")) {
                if (polygonMode.control)
                    return polygonMode.control.values;
                return useDynamic
                    ? polygons.concat([partialPolygon.concat([hoverPosition])])
                    : polygons.concat([partialPolygon]);
            }
            return [];
        };
        this.renderInputs = () => {
            if (!this.props.showInputs)
                return null;
            return (react_1.default.createElement("div", { className: "flex-container some-margin" },
                react_1.default.createElement("div", { className: "full-width" },
                    react_1.default.createElement("label", { className: "text pad-right" }, "Latitude:"),
                    react_1.default.createElement("input", { type: "number", value: this.state.lat, onChange: this.inputChange("lat"), className: "large-width input-field" })),
                react_1.default.createElement("div", { className: "full-width some-margin" },
                    react_1.default.createElement("label", { className: "text pad-right" }, "Longitude:"),
                    react_1.default.createElement("input", { type: "number", value: this.state.lng, onChange: this.inputChange("lng"), className: "large-width input-field" }))));
        };
        this.getInitialPickerMode = (props) => {
            if (props.pointMode)
                return "points";
            if (props.circleMode)
                return "circles";
            if (props.rectangleMode)
                return "rectangles";
            if (props.polylineMode)
                return "polylines";
            if (props.polygonMode)
                return "polygons";
            return "points";
        };
        this.handleClick = (e) => {
            const { precision, pointMode, circleMode, rectangleMode, polylineMode, polygonMode } = this.props;
            const { pickerMode, points, circles, circleCenter, rectangles, rectangleCorner, partialPolyline, partialPolygon } = this.state;
            const lat = utils_1.setPrecision(e.latlng.lat, precision);
            const lng = utils_1.setPrecision(e.latlng.lng, precision);
            this.setState({ lat, lng });
            switch (pickerMode) {
                case "points": {
                    if (pointMode && pointMode.control && pointMode.control.onClick) {
                        pointMode.control.onClick([lat, lng]);
                    }
                    else {
                        this.setState({ points: points.concat([[lat, lng]]) });
                    }
                    break;
                }
                case "circles": {
                    if (circleMode && circleMode.control && circleMode.control.onClick) {
                        circleMode.control.onClick([lat, lng]);
                    }
                    else {
                        if (circleCenter) {
                            const circle = {
                                center: circleCenter,
                                radius: utils_1.calculateRadius(circleCenter, [lat, lng])
                            };
                            this.setState({
                                circleCenter: null,
                                circles: circles.concat([circle])
                            });
                        }
                        else {
                            this.setState({ circleCenter: [lat, lng] });
                        }
                    }
                    break;
                }
                case "rectangles": {
                    if (rectangleMode &&
                        rectangleMode.control &&
                        rectangleMode.control.onClick) {
                        rectangleMode.control.onClick([lat, lng]);
                    }
                    else {
                        if (rectangleCorner) {
                            const rectangle = [rectangleCorner, [lat, lng]];
                            this.setState({
                                rectangleCorner: null,
                                rectangles: rectangles.concat([rectangle])
                            });
                        }
                        else {
                            this.setState({ rectangleCorner: [lat, lng] });
                        }
                    }
                    break;
                }
                case "polylines": {
                    if (polylineMode &&
                        polylineMode.control &&
                        polylineMode.control.onClick) {
                        polylineMode.control.onClick([lat, lng]);
                    }
                    else {
                        this.setState({
                            partialPolyline: partialPolyline.concat([[lat, lng]])
                        });
                    }
                    break;
                }
                case "polygons": {
                    if (polygonMode && polygonMode.control && polygonMode.control.onClick) {
                        polygonMode.control.onClick([lat, lng]);
                    }
                    else {
                        this.setState({
                            partialPolygon: partialPolygon.concat([[lat, lng]])
                        });
                    }
                }
            }
        };
        this.handleMouseMove = (e) => {
            const now = new Date().getTime();
            if (now - this.state.throttleTimer > 50) {
                const lat = utils_1.setPrecision(e.latlng.lat, this.props.precision);
                const lng = utils_1.setPrecision(e.latlng.lng, this.props.precision);
                this.setState({ hoverPosition: [lat, lng], throttleTimer: now });
            }
        };
        this.inputChange = (field) => (e) => {
            const newState = { [field]: Number(e.target.value) };
            this.setState(newState);
        };
        this.changeMode = (pickerMode) => () => {
            if (pickerMode !== this.state.pickerMode) {
                this.setState({
                    pickerMode,
                    circleCenter: null,
                    partialPolyline: [],
                    partialPolygon: [],
                    rectangleCorner: null
                });
            }
        };
        this.addPolyline = () => {
            this.setState({
                polylines: this.state.polylines.concat([this.state.partialPolyline]),
                partialPolyline: []
            });
        };
        this.addPolygon = () => {
            this.setState({
                polygons: this.state.polygons.concat([this.state.partialPolygon]),
                partialPolygon: []
            });
        };
        this.removeObject = (type) => (mapObject) => {
            const index = utils_1.indexOfObject(this.state[type], mapObject);
            const newState = Object.assign(this.state[type], []);
            newState.splice(index, 1);
            // @ts-ignore
            this.setState({ [type]: newState });
        };
        defaultState.pickerMode = this.getInitialPickerMode(props);
        this.state = defaultState;
    }
    render() {
        const { bindMap, mapStyle, useDynamic } = this.props;
        return (react_1.default.createElement(react_1.default.Fragment, null,
            this.renderBanner(),
            react_1.default.createElement(react_leaflet_1.Map, { style: mapStyle, className: "leaflet-crosshair", viewport: defaultViewport, maxBounds: bindMap ? mapBounds : undefined, maxBoundsViscosity: 1, onClick: this.handleClick, onMouseMove: useDynamic ? this.handleMouseMove : undefined, minZoom: 2, ref: "map" },
                this.renderTileLayer(),
                this.renderModeControl(),
                this.renderOverlays()),
            this.renderInputs()));
    }
}
LocationPicker.defaultProps = defaultProps;
exports.default = LocationPicker;
